# Copyright (C) 2001 by Dr. Dieter Maurer <dieter@handshake.de>
# D-66386 St. Ingbert, Eichendorffstr. 23, Germany
#
#			All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice and this permission
# notice appear in all copies, modified copies and in
# supporting documentation.
# 
# Dieter Maurer DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL Dieter Maurer
# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
# DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
# PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
# TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
'''MFolder - mirrowing folder

 Mirrowing folder allows to mirror one subfolder *M* to be
 seen (through acquisition) in the other subfolders.

 More precisely: If the request traverses an attribute *A* of the
 mirrowing folder along a subpath *P*
 leading to object *oa*,
 then the object *om* obtained by traversing along a maximal
 prefix of *P* starting in *M* is available in *oa*
 through acquisition.

 Example::

   'M/registration' is mirrowed in 'en/registration'

   'M/subscribe/news' is mirrowed in 'en/subscribe/news'.


 Use Case

   MFolder was developed to build a complex multilingual site.

   Of course, you want to separate language dependent content
   from language independent logic (and maybe language independent
   content), because the language dependent parts exist
   in variants for each language while the language independent
   parts exist only once.
   Nevertheless, the independent parts should be easily accessible
   from the language dependent parts -- **as if** they
   were in the same folders as they themselves.

   One possibility to achieve this would be to place all
   independent parts above the language folders.
   This would give an unstructured, difficult to understand
   and maintain folder containing all the logic.

   We could use transparent folders to alleviate the problem
   somewhat, but they suffer from name clash problems.
   If several objects with the same name exist in different
   folders, it is unclear, which object wins.

   With mirrowing folder, your language dependent folders
   (one for each language) and your logic folder
   all have the same structure. The mirrowing magic
   uses acquisition to map each subfolder of the logic
   folder onto the corresponding subfolder in the
   language folders. As usual with acquisition, the
   attributes of this mirrowed subfolder and its ancestors
   can be directly accessed. Other subfolders are
   not directly visible.

   Of course, this technique can be used whenever
   common objects should be easily used by various
   variants.


 Properties

   MirrowId__  -- the id of the subfolder to be mirrowed'''


from OFS.Folder import Folder, manage_addFolder
from Globals import InitializeClass, DTMLFile
from new import function
from HelpSupport import HelpSTXTopic_S


def getMFolderHelp():
  return HelpSTXTopic_S('MFolderHelp', 'Mirroring Folder Documentation',__doc__)


class MFolder(Folder):
  '''an MFolder behaves like a folder but it mirrows its
  subfolder given by 'MirrowId__' into all other subfolders.

  This is similar to 'TransparentFolder' but more selective
  and mirrowing is according to structure.'''

  meta_type= 'Mirrowing Folder'

  _properties= Folder._properties + (
    {'id': 'MirrowId__', 'type': 'string', 'mode' : 'w',},
    )

  manage_options=  (
    ({'label' : 'Contents', 'action':'manage_main',
      'help' : ('MFolder','MFolderHelp')},)
    + Folder.manage_options[1:]
    )


  MirrowId__= ''

  def __bobo_traverse__(self,request,entry_name):
    '''prepare mirrowing.'''
    mid= self.MirrowId__; mf= getattr(self,mid,None)
    path= getattr(request,'TraversalRequestNameStack',None)
    if entry_name != mid and mf and path is not None:
      path= path[:]
      context= mf; path.reverse()
      for s in path:
        try: ncontext= context.unrestrictedTraverse(s)
        except: break
        if not hasattr(ncontext,'__basicnew__'): break
        context= ncontext
      self= self.__of__(context)
    return getattr(self,entry_name)

InitializeClass(MFolder)

_code= manage_addFolder.func_code
_globals= manage_addFolder.func_globals.copy()
_globals['Folder']= MFolder
manage_addMFolder=function(_code,_globals,'manage_addMFolder',
                           manage_addFolder.func_defaults)

manage_addMFolderForm= DTMLFile('www/MFolderAdd', globals())




